# Genomic pipeline

## 0. Summary

This pipeline is based on data from Kelp from Fraser et al. (Nature Climate Change; referred thereafter as NCC), combined with two sets of new data new_data 1 and new_data 2 trimmed and demultiplexed independently. These 3 sets of data are then run together through the reference-based pipeline of Stacks. 


All scripts and metadata files needed to replicate the analysis from step 3 are included in this repository in the folder metadata_files_and_scripts.
The archived reads at the NCBI Short Read Archive (https://www.ncbi.nlm.nih.gov/sra) at project PRJNA802596 are the demultiplexed reads. They have the final names presented in the study. This pipeline start with the original samples names from the NCC publication. The translation table to match these scripts is inside translation.tsv the names under classic names are used until the final renaming step.

Please do not hesitate to contact dutoit.ludovic@gmail.com for any question/support. Note that sample names are different from the ones displayed in the paper until the very last step that rename then. 

## 1.Trimming

After data quality assessment using FastQC, we apply the same filtering to all three datasets:

* Trim adapters
* remove any reads with overall quality below 25
* remove any reads with homopolymers


```bash
module load cutadapt FastQC # Cluster specific

#published Fraser et al NCC

cutadapt -a AGATCGGAAGAGC -A AGATCGGAAGAGC  -m 76:76  -q 25 --overlap 5 -o temp1    -p  temp2 Kelp1_GBS_S1_R1_001.fastq.gz  Kelp1_GBS_S1_R2_001.fastq.gz # remove adapters
cutadapt -a  GGGGGGGGGGGGG  -A GGGGGGGGGGGGG  -m 76:76  -q 25 --overlap 5 -o trimmed_Kelp1_GBS_S1_R1_001.fastq    -p trimmed_Kelp1_GBS_S1_R2_001.fastq     temp1 temp2 # remove homopolymers
fastqc  trimmed_Kelp1_GBS_S1_R1_001.fastq trimmed_Kelp1_GBS_S1_R2_001.fastq
rm temp1 tremp2


fastqc *fastq

cd ../25Antarcticasamples/

### new data 1
cutadapt -a AGATCGGAAGAGC -A AGATCGGAAGAGC  -m 76:76  -q 25 --overlap 5 -o temp1    -p  temp2 GBS_Kelp_CF1_S1_R1_001.fastq.gz GBS_Kelp_CF1_S1_R2_001.fastq.gz # remove adapters
cutadapt -a  GGGGGGGGGGGGG  -A GGGGGGGGGGGGG  -m 76:76  -q 25 --overlap 5 -o trimmed_Kelp1_GBS_S1_R1_001.fastq    -p trimmed_GBS_Kelp_CF1_S1_R2_001.fastq     temp1 temp2 # remove homopolymers
fastqc  trimmed_Kelp1_GBS_S1_R1_001.fastq trimmed_Kelp1_GBS_S1_R2_001.fastq
rm temp1 temp2





### new data 2 on 2 lanes
mv CF2_S1_R* last_elahe/
cd last_elahe
cutadapt -a AGATCGGAAGAGC -A AGATCGGAAGAGC  -m 76:76  -q 25 --overlap 5 -o temp1    -p  temp2 CF2_S1_R1_001.fastq.gz  CF2_S1_R2_001.fastq.gz # remove adapters
cutadapt -a  GGGGGGGGGGGGG  -A GGGGGGGGGGGGG  -m 76:76  -q 25 --overlap 5 -o trimmed_petersl1_R1_sample.fq    -p trimmed_petersl1_R2_sample.fq      temp1 temp2 # remove homopolymers
rm temp1 temp2


cutadapt -a AGATCGGAAGAGC -A AGATCGGAAGAGC  -m 76:76  -q 25 --overlap 5 -o temp3    -p  temp4 lane2_S1_R1_001.fastq.gz  lane2_S1_R2_001.fastq.gz
cutadapt -a  GGGGGGGGGGGGG  -A GGGGGGGGGGGGG  -m 76:76  -q 25 --overlap 5 -o trimmed_petersl2_R1_sample.fq    -p trimmed_petersl2_R2_sample.fq      temp3 temp4 # remove homopolymers
rm temp3 temp4
```

## 2. Demultiplexing 

Dataset per dataset, but all the files end up in the folder samples, before concatenation of forward and reverse, and last base trimming.

**published Fraser et al NCC**


```bash
#barcodes are in : metadata/barcodes_published_NCC.txt
process_radtags -P -p raw_published_NCC/ -o ./samples/ -b barcodes_published_NCC.txt -e pstI -r -c -q --inline_inline # raw_published_NCC contain the trimmed data of Fraser et al. NCC 

#509690728 total sequences
# 83929748 barcode not found drops (16.5%)
#   105346 low quality read drops (0.0%)
#  1893956 RAD cutsite not found drops (0.4%)
#423761678 retained reads (83.1%)
```



**new data 1**

```bash
#barcodes are in metadata/newsamples_barcodes.txt
process_radtags -P -p raw_new/ -o ./samples/ -b newsamples_barcodes.txt -e pstI -r -c -q --inline_inline # raw_new contain the trimmed data of new_data 1 

#168090652 total sequences
#  7421710 barcode not found drops (4.4%)
#    26898 low quality read drops (0.0%)
#  1826782 RAD cutsite not found drops (1.1%)
#158815262 retained reads (94.5%)
```


**new data 2** 
```bash
process_radtags -P -p last_elahe/ -o ./samples/ -b last_elahebarcodes.txt -e pstI -r -c -q --inline_inline # last_elahe contain the trimmed data of new_data 2


#620412172 total sequences
#503949592 barcode not found drops (81.2%)
#    34440 low quality read drops (0.0%)
#  2836056 RAD cutsite not found drops (0.5%)
#113592084 retained reads (18.3%)
```
### Concatenation

I concatenate all the forward and reverse reads in one file per sample.

```python
#python
import os
#os.mkdir("samples_concat")
allfiles_to_concat = os.listdir("samples")
alltokeep=[]
with open("popmap_after_metadatacleaning.txt") as f:
  for line in f:
    print(line)
    samplename=line.split()[0]
    #print (samplename)
    checkfiles=[filename for filename in allfiles_to_concat if samplename+".r" in filename and filename.startswith(samplename)]
    if len (checkfiles)!=2: # that was weirdly complicated because some sample name are contained in others different ways, but the vcheck above solve it uysing the rem file
      print(line)
      raise Exception
    else:
      nonrem = [x.replace("rem.","")for x in checkfiles] # the 2 normal files at once
      for x in nonrem:
        assert os.path.exists("samples/"+x)
      os.system("zcat "+" samples/"+nonrem[0]+" samples/"+nonrem[1]+" samples/"+checkfiles[0]+" samples/"+checkfiles[0]+ " | gzip -c > samples_concat/"+samplename+".fq.gz" )


```

After adapter removal and barcode removal some of the forward and the reverse reads vary in length by 1 bp, creating unwanted length differences. I get rid of this problem by trimming everything to 68 bp ( from the 3' end) as they are either 69 or 68 after adapter removal.


```python
import os
if not os.path.exists("sample_concat68"): os.mkdir("sample_concat68")

i=0
for file in os.listdir("samples_concat"):
  i+=1
  print(i,file)
  os.system("cutadapt   -m 68 --length 68  -o sample_concat68/"+file+" samples_concat/"+file)
```

I remove Auck-10-J which is empty as well as end16.

	
## 3. SNP calling

First, variants will be identified in the attached samples only using the filtering parameters optimised by Fraser et al. NCC (-p 2; -r 0.2 --min_maf 0.2). Low quality samples with less than 5000 genotyped variant sites will be discarded and the filtering was performed again with the remaining high-quality individuals to rescue additional SNPs.

```
mkdir alignment
cd alignment
module load BWA SAMtools # Cluster specific
bwa index k15_t2_l5_a0.3_o0_e0.1.scaffolds.fa # available at https://repo.data.nesi.org.nz/
```

running the mapping with [align.sh](align.sh) with the following command iterated over each sample:

```
bwa mem -t 8 $bwa_db $src/${sample}.fq.gz  |   samtools view -b | samtools sort --threads 4 > ${sample}.bam
```


Then I run refmap from Stacks for quick run to identify low quality individuals using attached samples only.

```
#!/bin/sh
module load Stacks # Cluster specific
mkdir output_refmap
ref_map.pl --samples alignment --popmap popmap_after_metadatacleaning.txt -T 8  -o output_refmap
```

```
populations -P output_refmap/  -M popmap_attached.txt  --vcf -p 2 -r 0.2 -O clean --vcf --min-maf 0.2 
```

We identify low quality individuals (<5000 SNPs) 
```
# module load R-bundle-Bioconductor/3.13-gimkl-2020a-R-4.1.0
library("VariantAnnotation")
data<-readVcf("clean//populations.snps.vcf")


numbermissing<-function(x){
	return(length(grep("\\./\\.",x)))
}

countsofmissing<-apply(geno(data)$GT,2,numbermissing)
nonmissing<-dim( geno(data)$GT)[1]-countsofmissing

#which ones have less than X% of sites covered
print(sort(nonmissing))


toremove<- names(nonmissing[ which(nonmissing<5000)])

paste(toremove, collapse='\\s|^') # 42 inds (out of 202)
```
That identified 42 individuals that we exclude from the popmap to habe a clean run.
```
grep   -Ev "^auc.1\\s|^auc.3\\s|^bou7\\s|^bou1\\s|^bou6\\s|^bou9\\s|^bou2\\s|^bou10\\s|^bou3\\s|^cam20\\s|^ch1\\s|^cc1\\s|^cz4_JJ\\s|^cz19_BA\\s|^cz17_BA\\s|^cz34_BM\\s|^cz16_BA\\s|^cz18_BA\\s|^go17_G\\s|^mq.10\\s|^Mar.27_rooks\\s|^Mar.46_CD\\s|^Mar.47_CD\\s|^Mar.51_K\\s|^Mar.7_SP\\s|^Mar.9_SP\\s|^pat3.3\\s|^pat1.3\\s|^pat3.1\\s|^pat3.2\\s|^sg11_OH\\s|^sg13_OH\\s|^sg4_SA\\s|^thong2\\s|^thong5\\s|^thong8\\s|^thong4\\s|^thong7\\s|^thong6\\s|^thong1\\s|^thong3\\s|^Mar.4_Blow"  popmap_attached.txt > popmap_attached_clean.txt


populations -P output_refmap  -M popmap_attached_clean.txt  -p 2 -r 0.2 -O clean --vcf --min-maf 0.2
```


We use the first column that contains all the loci as a whitelist to apply  to all samples including raft/drit samples before filtering.

```
cut -f 1,2 clean/populations.snps.vcf  | grep -v "#" | sort | uniq > whitelist.txt
```

We then all loci, before selecting loci in the whitelit using vcftools:

```
populations -P output_refmap  -M popmap_after_metadatacleaning.txt -O clean --vcf  # include bad quality attached at this point (is removed later because still less than 5000, does not matter)
vcftools --vcf clean/populations.snps.vcf --positions whitelist.txt --recode --out allinds_whitelist
```

We identify low quality individuals (<5000 SNPs) again in R

```r
library("VariantAnnotation")
data<-readVcf("allinds_whitelist.recode.vcf")


numbermissing<-function(x){
	return(length(grep("\\./\\.",x)))
}

countsofmissing<-apply(geno(data)$GT,2,numbermissing)
nonmissing<-dim( geno(data)$GT)[1]-countsofmissing

#which ones have less than X% of sites covered
print(sort(nonmissing))
toremove<- names(nonmissing[ which(nonmissing<5000)])
length(toremove)
write(toremove, "toremove_after_whitelist.txt",sep='\n')
paste(toremove, collapse='\\s|^') # 38 inds (out of 247)

```



I take them off the popmap along with the attached low quality samples.

```sh
grep   -Ev "^auc.1\\s|^auc.3\\s|^bou7\\s|^bou1\\s|^bou6\\s|^bou9\\s|^bou2\\s|^bou10\\s|^bou3\\s|^cam20\\s|^ch1\\s|^cc1\\s|^cz4_JJ\\s|^cz19_BA\\s|^cz17_BA\\s|^cz34_BM\\s|^cz16_BA\\s|^cz18_BA\\s|^go17_G\\s|^mq.10\\s|^Mar.27_rooks\\s|^Mar.46_CD\\s|^Mar.47_CD\\s|^Mar.51_K\\s|^Mar.7_SP\\s|^Mar.9_SP\\s|^pat3.3\\s|^pat1.3\\s|^pat3.1\\s|^pat3.2\\s|^sg11_OH\\s|^sg13_OH\\s|^sg4_SA\\s|^thong2\\s|^thong5\\s|^thong8\\s|^thong4\\s|^thong7\\s|^thong6\\s|^thong1\\s|^thong3\\s|^Mar.4_Blow\\s|^ant_ant17\\s|^auc.1\\s|^auc.3\\s|^bou7\\s|^bou1\\s|^bou6\\s|^bou9\\s|^bou2\\s|^bou10\\s|^cc1\\s|^cz4_JJ\\s|^cz19_BA\\s|^cz17_BA\\s|^cz34_BM\\s|^cz16_BA\\s|^cz18_BA\\s|^go17_G\\s|^mq.10\\s|^Mar.27_rooks\\s|^Mar.46_CD\\s|^Mar.47_CD\\s|^Mar.51_K\\s|^Mar.7_SP\\s|^pat3.3\\s|^pat1.3\\s|^pat3.1\\s|^pat3.2\\s|^sg11_OH\\s|^tas2\\s|^thong2\\s|^thong5\\s|^thong8\\s|^thong6\\s|^thong1\\s|^Durv2_4\\s|^Durv2_2\\s|^Durv2_6\\s|^Mar.4_Blow"  popmap_after_metadatacleaning.txt > popmap_afterbadapple.txt
```

Those are the drift samples excluded:
[1] "ant_ant17" "tas2"      "Durv2_4"   "Durv2_2"   "Durv2_6"

I use vcftools to remove all the low quality samples from the vcf with all the SNPs:

```sh
cut -f 1 popmap_afterbadapple.txt > tokeep.txt
vcftools --vcf allinds_whitelist.recode.vcf  --keep tokeep.txt --recode  --out toconvert
```

SNP calling is done, we obtained 223,939 SNPs for 100 samples (198 independent individuals and 3 technical replicates of one Macquarie sample).


### 4. Phylogenetic tree 

We need to convert the VCF to hylip file. We'll do that using population but we need to create a popmap where each individual is its own
```r
#make a popmap from the individual in vcf
data<-readLines("toconvert.recode.vcf")
grep("#C",data)
#15
indline<-data[15]
inds<-unlist(strsplit(indline,"\t"))[10:length(unlist(strsplit(indline,"\t")))]#Obtain all individuals 
write.table(cbind(inds,inds),"popmap_cleaned_forphylip.txt",sep="\t",quote=F,row.names=F,col.names=F)
```

```bash
populations -V toconvert.recode.vcf  -M popmap_cleaned_forphylip.txt  -O clean    --phylip --vcf

module load Miniconda3 # Cluster specific
source activate iqtreetest
cd clean
grep "#" -v  toconvert.recode.p.fixed.phylip  >  populations.snps.p.fixed.cleaned.phylip
iqtree -nt 16 -s populations.snps.p.fixed.cleaned.phylip -st DNA -m GTR+G -bb 1000  -pre inferred -redo
# submitted in a file as: sbatch -A uoo00116 -t 23:00:00 --mem=64G -c 16 -J iqtree iqtree.sh
``


Rename the tree  to logical names with population suffix.

```bash
python rename_samples.py clean/inferred.contree
```

Rename again to have publications names.

```bash
python rename_samplesFINAL.py clean/inferredrenamed.contree
``` 

The output is Suppl_FigureS1_full_tree.contree attached in this repository.
